<?php
namespace Common\Respository;

use Backend\Model\CloudserverIpModel;
use Common\Respository\BusinessRespository;
use Common\Data\GiantAPIParamsData as GiantAPIParams;


/**
 * 交易仓储类
 * 可选择实现orderExtra()，whereExtra()，fieldsExtra()方法，
 * 为了有连表的情况出现，额外设置别名
 * @author: duanbin
 * @时间: 2016年10月9日 下午3:50:29
 */
class CloudserverBusinessRespository extends BusinessRespository{

	/**
	 * 扩充查询字段，在连表的情况下有别名使用
	 * @时间: 2016年10月14日 上午10:19:58
	 * @author: duanbin
	 * @param $fields
	 * @return array
	 */
	protected function fieldsExtra($fields){
		if(is_string($fields)){
			$fields = explode(',',$fields);
		}
		foreach($fields as $k => $filed){
			if($filed == 'product_name'){
				$fields[$k] = 'b.'.$filed;
			}elseif($filed == 'login_name'){
				$fields[$k] = 'c.'.$filed;
			}elseif($filed == 'ipaddress'){
				$fields[$k] = 'd.'.$filed;
			}else{
				$fields[$k] = $this->alias.'.'.$filed;
			}
		}
		return $fields;
	}
	
	/**
	 * 返回一个join数组
	 * 连表查询时，请实现此方法
	 * @时间: 2016年10月13日 下午6:14:18
	 * @author: duanbin
	 * @return array
	 */
	protected function joinsExtra(){
		return ['left join '.$this->table_prefix.'product as b on '.$this->alias.'.product_id = b.id','left join '.$this->table_prefix.'member as c on '.$this->alias.'.user_id = c.user_id','left join '.$this->table_prefix.'cloudserver_business_ip as d on '.$this->alias.'.api_bid = d.belong_server',];
	}
	
	/**
	 * 额外的扩充条件，这里要增加一项
	 * product_id值得条件
	 * @时间: 2016年10月13日 下午6:34:47
	 * @author: duanbin
	 * @param $where
	 * @param $request
	 * @return array
	 */
	protected function whereExtra($where,$request){
		$_where = [];
		foreach($where as $k => $v){
			if($k == 'product_name'){
				$_where['b.product_name'] = $v;
			}elseif($k == 'login_name'){
				$_where['c.login_name'] = $v;
			}elseif($k == 'ipaddress'){
				$_where['d.ipaddress'] = $v;
			}else{
				$_where[$this->alias.'.'.$k] = $v;
			}
		}
		return $_where;
	}
	
	/**
	 * 业务统计方法，统计出各个分类的总数等
	 * @时间: 2016年10月26日 上午11:59:06
	 * @author: duanbin
	 * @param $field
	 * @return array
	 */
	public function counter($field){
		$counter = [];
		/** 总记录数 */
		$counter['total'] = $this->queryBuilder($field,[]);
		/** 成功业务 */
		$where['a.state'] = ['eq',1];
		$counter['successful'] = $this->queryBuilder($field,$where);
		/** 已删除业务 */
		$where['a.state'] = ['eq',2];
		$counter['deleted'] = $this->queryBuilder($field,$where);
		/** 已过期业务 */
		$where['a.state'] = ['eq',3];
		$counter['expired'] = $this->queryBuilder($field,$where);
		/** 失败业务 */
		$where['a.state'] = ['eq',4];
		$counter['failed'] = $this->queryBuilder($field,$where);
		return $counter;
	}
	
	/**
	 * 同步云服务器的业务逻辑
	 * 方法内部已经对ptype进行了处理，会自动找出对应的ptype，不需要再传入了
	 * @时间: 2016年10月31日 下午5:05:36
	 * @author: duanbin
	 * @param $business_info
	 * @param string $attr
	 * @return bool
	 */
	public function CloudserverBusinessSynchronizing($business_info,$attr = 'api_bid',$access = 0){
		//如果你传入的是id值那么就获取信息，如果你传入的是一条业务记录(数组),那么就不用再去取了
		if(is_numeric($business_info) && !is_array($business_info)){
			//先获取本地业务信息
			$business_info_exists = $this->model()->where([$attr => ['eq',$business_info]])->find();
			//如果不存在这条记录，那么就写进去
			if(empty($business_info_exists)){
				$api_config = WebSiteConfig(2);//获取网站配置
				$api_access = $api_config["api_access"] + 0;
				$api_access = $access ? $access : ($api_access ? $api_access : 1);
				list($business_info,$temp) = [[],$business_info];
				$business_info['access'] = $api_access;
				$business_info[$attr] = $temp;
				$business_info['user_id'] = -1;
				$add_id = $this->model()->add($business_info);
				$business_info['id'] = $add_id;
			}else{
				$business_info = $business_info_exists;
			}
		}
		$api_access = $business_info["access"] + 0;
		$api_access = $api_access ? $api_access : 1;
		$business_info['business_id'] = $business_info[$attr];
		$id = $business_info['id'];
		//调用api同步业务信息
		$result = $this->syncBusinessFromZZIDC($business_info,GiantAPIParams::PTYPE_CLOUD_SERVER,GiantAPIParams::PNAME_CLOUDSERVER_SERVER);
		if(is_array($result) && !empty($result['json'])){
			//说明通用的单次同步业务逻辑未出错；
			//拿取api返回的信息
			$product_info = $result['product_info'];
			$bus_info = $result['json'] ['info'];
			//处理相应的信息，拼凑要更新到本地的数据
			// 业务信息
			$parms['port'] = $bus_info['port'];
			$parms['area_code'] = $bus_info['server_areacode'];
			$parms['nw_ip'] = $bus_info['nwip'];
			$parms['ip_bid'] = $bus_info['ip_ywbh'];
			$parms['cloudhost_password'] = $bus_info['password'];
			$parms['input_name'] = $bus_info['input_name'];
			$parms['memory'] = $bus_info['memory'];
			$parms['disk'] = $bus_info['ssddisk'];
			$parms['overdue_time'] = $bus_info['overDate']; // 到期时间
			$parms['cloudserver_user'] = $bus_info['username'];
			$parms['buy_time'] = $bus_info['gmqx'];
			$parms['cpu'] = $bus_info['cpucount'];
			$parms['create_time'] = $bus_info['createDate']; // 创建时间
			$parms['api_bid'] = $bus_info['ywbh'];
			$parms['os_type'] = $bus_info['sysTem'];
			$parms['service_code'] = $bus_info['serviceCode'];
			$parms['service_code_psd'] = $bus_info['servicepasswd'];
			$parms['access'] = $api_access;
			//这里在顺便同步一下快云服务器的ip，如果有的话
			if($bus_info['ip_ywbh'] != 'excep' && $bus_info['ip_ywbh'] != null){
				$parms ['ip_bid'] = $bus_info['ip_ywbh'];
				$parms['ip_state'] = '0';
				//这里处理下ip的同步
				$cloudserber_ip_business = new CloudserverIpBusinessRespository(new CloudserverIpModel());
				$ip_sync_res = $cloudserber_ip_business->CloudserverIpBusinessSynchronizing($bus_info['ip_ywbh'],$business_info['business_id']);
				if($ip_sync_res === false){
					$this->model()->setError('服务器繁忙，ip地址同步失败，请重试');
					return false;
				}
			}else{
				$parms['ip_state'] = '2';
			}
			$parms['sync_state'] = '1';
			$parms['product_name'] = '快云服务器';
			$parms['product_id'] = '214';
			$parms['user_id'] = !$business_info['user_id'] ? '-1' : $business_info['user_id'];
			$parms['login_name'] = !$business_info['login_name'] ? '待转让会员' : $business_info['login_name'];
			//这里更新的是cloudspace业务表的信息
			$cloudspace_update_res = $this->model()->where(['id' => ['eq',$id]])->save($parms);
			if($cloudspace_update_res === false){
				$this->model()->setError('服务器繁忙，同步失败，请重试');
				return false;
			}else{
				return true;
			}
		}else{
			//说明同步调用api出错了,将错误结果返回出去
			$this->model()->setError(business_code($result));
			return FALSE;
		}
	}
	
	/**
	 * 删除云服务器的方法
	 * @时间: 2016年11月4日 下午2:32:16
	 * @author: duanbin
	 * @param $id
	 * @return bool
	 */
	public function eraseCloudserverBusiness($id){
		return $this->erase($this->firstOrFalse(['id' => ['eq',$id]]),GiantAPIParams::PTYPE_CLOUD_SERVER,'cloudserver.server');
	}
	
}