<?php
namespace Frontend\Model;

use Common\Data\StateData as SD;
use Frontend\Model\BusinessModel;
use Common\Aide\AgentAide;
use Common\Data\GiantAPIParamsData as GD;

class DomainModel extends BusinessModel{
	protected $trueTableName = 'agent_domain_business';

	public function conver_par(){
		$date = request();
		$where["bs.user_id"] = array("eq",session("user_id"));
		if(!empty($date['select'])){
			$select = clearXSS($date['select']);
			$map['bs.api_bid'] = array("like","%$select%");
			$map['_logic'] = 'OR';
			$map['bs.domain_name'] = array("like","%$select%");
			$map['_logic'] = 'OR';
			$map["bs.beizhu"] = array("like","%$select%");
			$where['_complex'] = $map;
		}
		return $where;
	}

	/**
	 * 跳转中国数据控制面板
	 * @date: 2016年10月29日 下午2:52:54
	 * @author: Lyubo
	 * @return int|mixed
	 */
	public function domain_panel(){
		$data = request();
		$api_bid = $data["business_id"];
		$user_id = session("user_id");
		$info = $this->get_api_bid_one($api_bid);
		$api_config = WebSiteConfig(2);//获取网站配置
		$api_access = $api_config["api_access"] + 0;
		$api_access = $api_access ? $api_access : 1;
		$agent = new AgentAide(1,$api_access);
		$transaction = $agent->servant;
		if($info){
			try{
				$result = $transaction->domainskip($api_bid);
				$log_message = "ptype:".GD::PTYPE_DOMAIN.'||tid=15';
				api_log($user_id,$log_message,$result,"跳转中国数据控制面板");
				$json = json_decode($result,true);
			}catch(\Exception $e){
				$log_message = "ptype:".GD::PTYPE_DOMAIN.'||tid=15';
				if(empty($result)){
					$result = $e->getMessage();
				}
				api_log($user_id,$log_message,$result,"跳转中国数据控制面板失败");
				return -9;
			}
			return $json;
		}
	}

	/**
	 * 域名产品信息
	 * @date: 2016年11月20日 下午1:54:22
	 * @author: Lyubo
	 * @return: array
	 */
	public function domain_product(){
		$product = new ProductModel();
		$product_type_list = $product->get_type_list('4');//4为域名的顶级分类
		foreach($product_type_list as $ptl){
			$product_list[$ptl['api_ptype']] = $product->domain_info($ptl ['id']);
		}
		return $product_list;
	}

	/**
	 * 默认选中域名  获取默认域名信息
	 * @date: 2016年11月20日 下午4:52:42
	 * @author: Lyubo
	 * @return mixed
	 */
	public function get_default_domain(){
		$product = new ProductModel();
		// 获取信息
		$result = WebSiteConfig(SD::SYS_DEFAULT_DOMAIN);
		// 以逗号分隔
		$value = explode(",",$result["default_domain"]);
		foreach($value as $key => $val){
			$default_domain[$key] = $product->get_product($val);
		}
		return $default_domain;
	}

	/**
	 * 域名查询
	 * @date: 2016年11月21日 上午11:46:38
	 * @author: Lyubo
	 * @param $domain_type	-域名类型（中文/英文）
	 * @param $product_type	-域名类型（中文/英文）
	 * @param $domain		-域名名称
	 * @param $domain_id	-域名id
	 * @return mixed
	 */
	public function doamin_query($domain_type,$product_type,$domain,$domain_id){
		$product = new ProductModel();
		$ids = "";
		$suffix = [];
		$where['id'] = ['eq',$domain_id];
		$where['product_type_id'] = ['eq',$product_type];
		$where["_string"] = "FIND_IN_SET('2',stage)";
		$check_suffix_info = $product->where($where)->find();
		if(empty($check_suffix_info)){
			return -2;
		}
		$id = $check_suffix_info["id"];
		$product_list = $product->product_by_id($id,$domain_type);
		if(!is_array($product_list)){
			return -2;
		}
		$suffix[] = $product_list["product_name"];
		$api_config = WebSiteConfig(2);//获取网站配置
		$api_access = $api_config["api_access"] + 0;
		$api_access = $api_access ? $api_access : 1;
		$agent = new AgentAide(1,$api_access);
		$transaction = $agent->servant;
		try{
			$result = $transaction->new_domain_quary($domain,$suffix);
//			api_log(-1,$domain."域名查询",$result,"查询成功");
		}catch(\Exception $e){
			api_log(-1,$domain."域名查询",$e->getMessage(),"查询失败");
			return -9;
		}
		$xstr = json_decode($result,true);
		if($xstr["code"] != 0){
			return $xstr["code"];
		}
		//可以注册列表
		$yesList = $xstr["info"]["yesList"];
		//查询失败列表
		$failList = $xstr["info"]["failList"];
		//不可注册列表
		$noList = $xstr["info"]["noList"];
		//修改状态
		unset($product_list['config']);
		$product_name = $product_list["product_name"];
		//可以注册
		if(in_array($product_name,$yesList)){
			$product_list["is_reg"] = 1;
		}
		//查询失败
		if(in_array($product_name,$failList)){
			$product_list["is_reg"] = 0;
		}
		//不可以注册
		if(in_array($product_name,$noList)){
			$product_list["is_reg"] = 2;
		}
		$product_list["domain_name"] = $domain;
		$product_list["div_id"] = $domain.str_replace(".","",str_replace(".","",$product_name));
		return $product_list;
	}

	/**
	 * 显示域名注册
	 * @date: 2016年11月27日 上午11:11:48
	 * @author: Lyubo
	 * @param $member_id
	 * @param $data
	 * @return int|mixed
	 */
	public function show_domain_register($member_id,$data){
		$product_id = $data["product_id"];
		$domain_type = $data["api_ptype"];
		$domain = $data["domain"];
		$product = new ProductModel();
		//获取最近一条联系人信息
		$contact = $this->get_recently_contact($member_id);
		//获取域名产品信息
		$info = $product->get_product($product_id);
		if(empty($info)){
			return -2;
		}
		if(GD::PTYPE_DOMAIN !== $info["api_name"]){
			return -2;
		}
		//查询域名是否注册
		/*$query = $this->check_domain($domain_type,$domain,$info["id"]);
		if($query !== -1){
			return $query;
		}*/
		//封装信息
		$params["contact"] = $contact;
		$params["info"] = $info;
		$params["reg_domain"] = $domain;
		return $params;
	}

	/**
	 * 显示域名注册
	 * @date: 2016年11月27日 上午11:11:48
	 * @author: Lyubo
	 * @param $domain_type
	 * @param $domain
	 * @param $ids
	 * @return bool
	 */
	public function check_domain($domain_type,$domain,$id){
		$product = new ProductModel();
		$product_info = $product->product_by_id($id,$domain_type);
		if(!$product_info){
			return -2;
		}
		if($product_info["api_name"] !== GD::PTYPE_DOMAIN){
			return -2;
		}
		$suffix[0] = $product_info["product_name"];
		$api_config = WebSiteConfig(2);//获取网站配置
		$api_access = $api_config["api_access"] + 0;
		$api_access = $api_access ? $api_access : 1;
		$agent = new AgentAide(1,$api_access);
		$transaction = $agent->servant;
		try{
			$result = $transaction->new_domain_quary($domain,$suffix);
			api_log(-1,$domain.$suffix[0]."域名查询",$result,"查询成功");
		}catch(\Exception $e){
			api_log(-1,$domain.$suffix[0]."域名查询",$e->getMessage(),"查询失败");
			return -9;
		}
		$xstr = json_decode($result,true);
		if($xstr["code"] != 0){
			return $xstr["code"];
		}
		//可以注册列表
		$yesList = $xstr["info"]["yesList"];
		//查询失败列表
		$failList = $xstr["info"]["failList"];
		//不可注册列表
		$noList = $xstr["info"]["noList"];
		//修改状态
		$product_name = $product_info["product_name"];
		//可以注册
		if(in_array($product_name,$yesList)){
			return -1;
		}
		//查询失败
		if(in_array($product_name,$failList)){
			return -43;
		}
		//不可以注册
		if(in_array($product_name,$noList)){
			return -42;
		}
		return -43;
	}

	/**
	 * 获取最近1条联系人信息
	 * @date: 2016年11月27日 上午11:16:07
	 * @author: Lyubo
	 * @param $member_id
	 * @return mixed
	 */
	public function get_recently_contact($member_id){
		$domain_contact = M("domain_contact");
		$where['user_id'] = ["eq",$member_id];
		return $domain_contact->field("id,domain_business_id,user_id,create_time,dwmc,dwmcYW,zcrX,zcrM,zcrGJ,zcrSF,zcrCS,zcrDZ,zcrDZ2,zcrYZBM,zcrYX,zcrDH,zcrCZ,zjhm")
							  ->where($where)->order("create_time DESC")->find();
	}

	/**
	 * 域名注册调用接口
	 * @date: 2016年11月27日 下午1:48:21
	 * @author: Lyubo
	 * @param $product_id
	 * @param $domain
	 * @param $member_id
	 * @param $buy_time
	 * @param $buy_num
	 * @param $contact
	 * @param $api_type
	 * @return int
	 */
	public function domain_register($product_id,$domain,$member_id,$buy_time,$buy_num,$contact,$api_type){
		$api_config = WebSiteConfig(2);//获取网站配置
		$api_access = $api_config["api_access"] + 0;
		$api_access = $api_access ? $api_access : 1;
		$product = new ProductModel();
		$order = new OrderModel();
		//添加联系人表
		$contact ["user_id"] = $member_id;
		$contact ["create_time"] = date('Y-m-d H:i:s');
		$domain_contact = M("domain_contact");
		$add_contact = $domain_contact->add($contact);
		if(!$add_contact){
			return -17;//添加联系人信息失败
		}
		$contact_id = $domain_contact->getLastInsID();
		$params ["user_id"] = $member_id;
		$params ["product_id"] = $product_id;
		$params ["free_trial"] = 0;
		$params ["order_time"] = $buy_time*12;
		$params ["order_quantity"] = $buy_num;
		$params['access'] = $api_access;
		// 获取产品信息
		$product_info = $product->get_product($product_id);
		if(!$product_info || $product_info ["product_state"] == 0){
			return -2;//产品信息获取失败
		}
		//拼接域名
		$domain = $domain.$product_info ["product_name"];
		//录入订单
		$order_id = $order->order_do_entry($params,$member_id,SD::NEW_ORDER,$product_info,null,$domain,$api_type);
		if($order_id <= 0){
			return $order_id;
		}
		$agent = new AgentAide(1,$api_access);
		$transaction = $agent->servant;
		/**调用接口，域名注册*/
		try{
			$api_contact = $this->api_contact($contact);//处理联系人信息为API参数
			if($api_type == 1 || $api_type == 100012){
				$api_type = 100012;
			}else{
				$api_type = 5;
			}
			$result = $transaction->domainregistration($domain,$api_contact,$buy_time*12,$api_type);
			//修改订单失败，默认失败，添加业务表信息成功时再修改成功
			$order_data['state'] = SD::FAILURE_ORDER;
			$order->order_edit($order_id,$order_data);
			$log_message = '域名注册，域名：['.$domain.']';
			api_log($member_id,"ptype:".GD::PTYPE_DOMAIN."--pname:".GD::PNAME_DOMAIN_REGISTRATION."--tid:".GD::TID_DOMAIN,$result,$log_message);
		}catch(\Exception $e){
			$log_message = '域名注册，域名：['.$domain.']接口调用失败失败';
			api_log($member_id,"ptype:".GD::PTYPE_DOMAIN."--pname:".GD::PNAME_DOMAIN_REGISTRATION."--tid:".GD::TID_DOMAIN,$e->getMessage(),$log_message);
			return -9;
		}
		$xstr = json_decode($result,true);
		if($xstr['code'] != 0){
			return $xstr['code'];
		}
		/**调用接口，获取业务信息*/
		try{
			$result = $transaction->getdomaininfo($xstr["info"]["busiId"]);
			$log_message = '获取域名业务信息成功';
			api_log($member_id,"ptype:".GD::PTYPE_DOMAIN."--pname:".GD::PNAME_DOMAIN_LIST."--tid:".GD::TID_DOMAIN,$result,$log_message);
			$json = json_decode($result,true);
			if($json['code'] != 0){
				return $json['code'];
			}
		}catch(\Exception $e){
			$log_message = '域名注册成功,添加域名业务信息失败';
			api_log($member_id,"ptype:".GD::PTYPE_DOMAIN."--pname:".GD::PNAME_DOMAIN_LIST."--tid:".GD::TID_DOMAIN,$e->getMessage(),$log_message);
			return -122;
		}
		//域名业务列表
		$domain_list = $json['info'];
		//域名业务编号
		$domain_business_id = null;
		$domain_order_id = $domain_list["orderId"];
		$domain_psd = $domain_list["domainPwd"];
		$parms["user_id"] = $member_id;
		$parms['product_id'] = $product_id;
		$parms['create_time'] = $domain_list['createDate'];
		$parms['overdue_time'] = $domain_list['overDate'];
		$parms['api_bid'] = $domain_list['api_bid'];
		$parms['product_name'] = $domain_list['product_name'];
		$parms['domain_name'] = $domain_list['domain_name'];
		$parms['access'] = $api_access;
		$date = date('Y-m-d H:i:s');
		if($domain_list['overDate'] < $date){
			$parms['state'] = '0';
		}else{
			$parms['state'] = '1';
		}
		$parms['mm'] = $domain_psd;
		//1:西部数码3：万网4：新网5：中国数据6：新网互联7:ResellerClub
		if($domain_list['api_type'] == 100004){
			$parms['provider'] = '新网互联';
			$parms['api_type'] = 6;
		}else if($domain_list['api_type'] == 100006){
			$parms['provider'] = 'ResellerClub';
			$parms['api_type'] = 7;
		}else if($domain_list['api_type'] == 100012){
			$parms['provider'] = '西部数码';
			$parms['api_type'] = 1;
		}else{
			$parms['api_type'] = $domain_list['api_type'];
			if($domain_list['api_type'] == 3){
				$parms['provider'] = '万网';
			}else if($domain_list['api_type'] == 4){
				$parms['provider'] = '新网';
			}else if($domain_list['api_type'] == 5){
				$parms['provider'] = '中国数据';
			}
		}
		$parms['fdnsym'] = $domain_list['fdnsym'];
		$parms['zdnsym'] = $domain_list['zdnsym'];
		$add_domain = $this->add($parms);
		if(!$add_domain){
			return -18;
		}
		$domain_business_id = $this->getLastInsID();
		if($domain_business_id != null){
			//修改订单成功
			$order_data['state'] = SD::SUCCESS_ORDER;
			$order_data['api_id'] = $domain_order_id;
			$order->order_edit($order_id,$order_data);
			//修改域名业务ID信息
			$upd_contact['domain_business_id'] = $domain_business_id;
			$contact_where['id'] = ["eq",$contact_id];
			$this->update_domain_contact($contact_where,$upd_contact);
			//删除没有域名业务ID的联系人信息
			$del_contact['user_id'] = ['eq',$member_id];
			$del_contact['domain_business_id'] = ['exp','is null'];
			$this->delete_domain_contact($del_contact);
			return -1;
		}
		return -15;
	}

	/**
	 * 处理联系人为API参数
	 * @date: 2016年11月27日 下午2:31:39
	 * @author: Lyubo
	 * @param $contact
	 * @return string
	 */
	private function api_contact($contact){
		$domainparams = "";
		$domainparams .= "dwmc:".$contact ["dwmc"];
		$domainparams .= "-dwmcYW:".$contact ["dwmcYW"];
		$domainparams .= "-zcrX:".$contact ["zcrX"];
		$domainparams .= "-zcrM:".$contact ["zcrM"];
		$domainparams .= "-zcrGJ:".$contact ["zcrGJ"];
		$domainparams .= "-zcrSF:".$contact ["zcrSF"];
		$domainparams .= "-zcrCS:".$contact ["zcrCS"];
		$domainparams .= "-zcrDZ:".$contact ["zcrDZ"];
		$domainparams .= "-zcrDZ2:".$contact ["zcrDZ2"];
		$domainparams .= "-zcrYZBM:".$contact ["zcrYZBM"];
		$domainparams .= "-zcrYX:".$contact ["zcrYX"];
		$domainparams .= "-zjhm:".$contact ["zjhm"];
		$domainparams .= "-zcrDH:".strtr($contact ["zcrDH"],"-","+"); //-号替换为+号
		$domainparams .= "-isPersonal:".$contact ["isPersonal"];
		$domainparams .= "-zcrName:".$contact ["zcrName"];
		$domainparams .= "-zcrNameEn:".$contact ["zcrNameEn"];
		$domainparams .= "-zcrYX:".$contact ["zcrYX"];
		$domainparams .= "-isPersonal:".$contact ["isPersonal"];
		$domainparams .= "-zcrName:".$contact ["zcrName"];
		$domainparams .= "-zcrNameEn:".$contact ["zcrNameEn"];
		$domainparams .= "-zcrYX:".$contact ["zcrYX"];
		return $domainparams;
	}

	/**
	 * 域名续费展示
	 * @date: 2016年12月3日 上午11:41:25
	 * @author: Lyubo
	 * @param: $domain_id,$member_id
	 * @return: $domain_info
	 */
	public function domain_renewals_info($domain_id,$member_id){
		$where["db.id"] = ["eq",$domain_id];
		$where["db.user_id"] = ["eq",$member_id];
		return $this->alias(' db ')
					->field("db.id,domain_name,db.user_id,db.create_time,db.overdue_time,api_bid,provider,state,product_id,db.product_name, api_type,db.access,p.product_type_id")
					->join("left join ".C('DB_PREFIX')."product p on (p.id=db.product_id)")->where($where)->find();
	}

	/**
	 * 域名续费
	 * @date: 2016年12月3日 下午3:19:20
	 * @author: Lyubo
	 * @param: $domain_id,$member_id,$renewals_time
	 * @return boolean
	 */
	public function doamin_renewals($domain_id,$member_id,$renewals_time){
		$product = new ProductModel();
		$order = new OrderModel();
		$where['id'] = ["eq",$domain_id];
		$where["user_id"] = ["eq",$member_id];
		$domain_info = $this->where($where)->find();
		//获取不到业务信息或业务状态错误
		if(!$domain_info || $domain_info ["state"] != 1){
			return -10;
		}
		//计算过期时间
		$over_day = overdue_day($domain_info["overdue_time"]);
		if($over_day < -31){
			return -36;
		}
		//获取不到接口业务编号
		if(!$domain_info ["api_bid"]){
			return -11;
		}
		// 获取产品信息
		$product_info = $product->get_product($domain_info ["product_id"]);
		if(!$product_info || $product_info ["product_state"] == 0){
			return -2;
		}
		$api_access = $domain_info["access"] + 0;
		$api_access = $api_access ? $api_access : 1;
		$params ["product_id"] = $domain_info["product_id"];
		$params ["order_time"] = $renewals_time;
		$params ["order_quantity"] = 1;
		$params["business_id"] = $domain_id;
		$dataarray = array(
			"user_id" => $member_id,
			"order_type" => SD::RENEWALS_ORDER, // 续费
			"state" => SD::FAILURE_ORDER,
			"ip_address" => $domain_info['domain_info'],
			"product_type" => $product_info['product_type_id'],
			"product_id" => $product_info['id'],
			"product_name" => $product_info['product_name'],
			"free_trial" => 0, // 不是试用
			"order_time" => $renewals_time, // 续费12个月
			"order_quantity" => 1,
			"business_id" => $domain_id,
			"create_time" => current_date(),
			"login_name" => $domain_info['login_name'],
			"complete_time" => current_date(),
			'order_log' => '会员'.$member_id."续费".$domain_info['domain_name'].'域名',
			"access" => $api_access
		);
		//录入订单
		$product_type_info['business_id'] = $domain_info['id'];
		if($domain_info["api_type"] == 1 || $domain_info["api_type"] == 100012){
			$api_type = 100012;
		}else{
			$api_type = 5;
		}
		$order_id = $order->order_do_entry($dataarray,$domain_info["user_id"],SD::RENEWALS_ORDER,$product_info,null,null,$api_type,$domain_info['id']);
		if($order_id <= 0){
			return $order_id;
		}
		$json = null;
		$agent = new AgentAide(1,$api_access);
		$transaction = $agent->servant;
		$ptype = GD::PTYPE_DOMAIN;
		$pname = GD::PNAME_DOMAIN_RENEWALS;
		$tid = GD::TID_DOMAIN;
		$bid = $domain_info["api_bid"];
		$yid = $renewals_time;
		try{
			/*调用接口，域名续费*/
			$result = $transaction->domainrenewals($bid,$yid);
			$json = json_decode($result,true);
			api_log($member_id,"ptype:".$ptype."--pname:".$pname."--tid:".$tid."--bid:".$bid."--yid:".$yid,$dataarray["order_log"],"域名续费");
		}catch(\Exception $e){
			api_log($member_id,"ptype:".$ptype."--pname:".$pname."--tid:".$tid."--bid:".$bid."--yid:".$yid,$e->getMessage(),"域名续费失败—接口调用失败");
			//修改订单状态
			$order_data["state"] = SD::FAILURE_ORDER;
			$order_data["order_log"] = $dataarray["order_log"]."-接口调用失败";
			$order->order_edit($order_id,$order_data);
			return -9;
		}
		if($json == null){
			return -9;
		}
		//返回API code
		if($json['code'] != 0){
			return $json['code'];
		}
		/*域名业务续费时间*/
		$ren = $this->domain_renewals($domain_id,$renewals_time);
		if($ren === false){
			return -12;
		}
		//修改订单成功
		$order_data["state"] = SD::SUCCESS;
		$order->order_edit($order_id,$order_data);
		return -1;
	}

	/**
	 * 域名续费
	 * @param $domain_id
	 * @param $month
	 * @return false|int
	 */
	public function domain_renewals($domain_id,$month){
		return $this->execute("UPDATE `agent_domain_business` SET `overdue_time`= DATE_ADD(overdue_time, INTERVAL ".$month." MONTH) WHERE `id` = ".$domain_id,false);
	}

	/**
	 * 修改域名联系人表business_id
	 * @date: 2016年11月27日 下午3:21:29
	 * @author: Lyubo
	 * @param: $contact_id，$upd_contact
	 * @return boolean
	 */
	public function update_domain_contact($contact_where,$upd_contact){
		$domain_contact = M("domain_contact");
		return $domain_contact->where($contact_where)->save($upd_contact);
	}

	/**
	 * 删除没有域名业务ID的联系人信息
	 * @date: 2016年11月27日 下午3:35:30
	 * @author: Lyubo
	 * @param: $member_id，$del_contact
	 * @return boolean
	 */
	public function delete_domain_contact($del_contact){
		$domain_contact = M("domain_contact");
		return $domain_contact->where($del_contact)->delete();
	}
}